import 'dart:developer';
import 'dart:io';

import 'package:video_confi/constants.dart';
import 'package:video_confi/screens/login/login.dart';
import 'package:video_confi/screens/metting/room/room_meeting.dart';
import 'package:video_confi/screens/metting/room/room_modules.dart';
import 'package:video_confi/screens/welcome/welcome.dart';
import 'package:video_confi/features/signaling/room_client_repository.dart';
import 'package:video_confi/features/me/bloc/me_bloc.dart';
import 'package:video_confi/features/media_devices/bloc/media_devices_bloc.dart';
import 'package:video_confi/features/peers/bloc/peers_bloc.dart';
import 'package:video_confi/features/producers/bloc/producers_bloc.dart';
import 'package:video_confi/features/room/bloc/room_bloc.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:random_string/random_string.dart';

import 'package:flutter/material.dart';

import 'app_modules/app_modules.dart';
import 'models/room_model.dart';

class DevHttpOverrides extends HttpOverrides {
  @override
  HttpClient createHttpClient(SecurityContext? context) {
    return super.createHttpClient(context)
      ..badCertificateCallback =
          (X509Certificate cert, String host, int port) => true;
  }
}

void main() {
  HttpOverrides.global = new DevHttpOverrides();
  runApp(
    MultiBlocProvider(
      providers: getAppModules(),
      child: VideoConfi(),
    ),
  );
}

class VideoConfi extends StatelessWidget {
  var url;

  VideoConfi({Key? key, this.url}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    HttpOverrides.global = DevHttpOverrides();
    return MultiBlocProvider(
      providers: getAppModules(),
      child: MaterialApp(
        debugShowCheckedModeBanner: false,
        initialRoute: '/room',
        theme: Constants.theme,
        // ignore: missing_return
        onGenerateRoute: (settings) {
          if (settings.name == Welcome.RoutePath) {
            return MaterialPageRoute(
              builder: (context) => LoginPage(),
            );
          }

          RoomModel model = RoomModel('sandeep', 'sbppls05');
          if (settings.name == RoomMeeting.RoutePath) {
            return MaterialPageRoute(
              builder: (context) => MultiBlocProvider(
                  providers: getRoomModules(settings: settings),
                  child: RepositoryProvider(
                    lazy: false,
                    create: (context) {
                      final meState = context.read<MeBloc>().state;
                      String displayName = meState.displayName;
                      String id = meState.id;
                      final roomState = context.read<RoomBloc>().state;
                      String url = roomState.url;

                      Uri? uri = Uri.parse(url);
                      log('URI: $uri');

                      return RoomClientRepository(
                        peerId: id,
                        displayName: displayName,
                        url: 'wss://letsmeet.no',
                        roomId: uri.queryParameters['roomId'] ??
                            uri.queryParameters['roomid'] ??
                            randomAlpha(8).toLowerCase(),
                        peersBloc: context.read<PeersBloc>(),
                        producersBloc: context.read<ProducersBloc>(),
                        meBloc: context.read<MeBloc>(),
                        roomBloc: context.read<RoomBloc>(),
                        mediaDevicesBloc: context.read<MediaDevicesBloc>(),
                      )..join();
                    },
                    child: RoomMeeting(roomModelArguments: model),
                  )),
            );
          }
        },
      ),
    );
  }
}
